<?php
namespace Shopware\trus1TrustedShopsBadge\Subscriber;

use DateTime;
use Enlight\Event\SubscriberInterface;

use Exception;
use Shopware\trus1TrustedShopsBadge\Components\Configuration;
use Shopware\Models\Shop\Shop;

/**
 * Class Frontend
 * @package Shopware\trus1TrustedShopsBadge\Subscriber
 * @author    Trusted Shops GmbH
 * @copyright 2017 Trusted Shops GmbH
 * @license   http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 * @link      http://www.trustedshops.de/
 */
class Frontend implements SubscriberInterface
{
    /**
     * @var Configuration
     */
    private $tsConfig;

    /**
     * @var $pathVar
     */
    private $pathVar;

    /**
     * @var $template
     */
    private $template;

    /**
     * @var \Shopware\Models\Shop\Shop
     */
    private $shop;

    /**
     * Frontend constructor.
     * @param Configuration $config
     * @param $pathVar
     * @param \Enlight_Template_Manager $template
     */
    public function __construct(Configuration $config, $pathVar, \Enlight_Template_Manager $template, Shop $shop) {
        $this->tsConfig = $config;
        $this->pathVar = $pathVar;
        $this->template = $template;
        $this->shop = $shop;

    }
    /**
     * Returns an array of event names this subscriber wants to listen to.
     *
     *
     * @return array The event names to listen to
     */
    public static function getSubscribedEvents()
    {
        return array(
            'Enlight_Controller_Action_PostDispatchSecure_Frontend' => 'onFrontendPostDispatch',
            'Enlight_Controller_Action_PostDispatchSecure_Frontend_Checkout' => 'onCheckoutPostDispatch'
        );
    }

    /**
     * @param \Enlight_Event_EventArgs $args
     */
    public function onCheckoutPostDispatch(\Enlight_Event_EventArgs $args)
    {
        /** @var \Shopware_Controllers_Frontend_Checkout $subject */
        $subject = $args->getSubject();
        $view = $subject->View();
        $request = $subject->Request();

        if ($request->getActionName() == 'finish') {
            $basket = $view->getAssign('sBasket');
            $parentProductSKUs = array();

            foreach ($basket['content'] as &$basketItem) {

                $basketItem['dummy'] = 1;
                $article = Shopware()->Models()->getRepository('Shopware\Models\Article\Detail')->findBy(
                    array('articleId' => $basketItem['articleID'], 'kind' => 1)
                );
                foreach ($article as $articleObj) {
                    $parentProductSKUs[$basketItem['ordernumber']] = $articleObj->getNumber();
                }
            }
            $view->assign('basketItemsAdditionalInfos', $parentProductSKUs); // Mapping from variant product article Id to parent order number
        }
    }

    /**
     * Calculate delivery date for basket
     *
     * @param array $basket
     * @return string
     */
    protected function getDeliveryDate($basket)
    {
        $deliveryDays = $this->getMaxDeliveryTime($basket);

        if(date('N', strtotime("+$deliveryDays days")) == 7) {
            //if calculated delivery time is Sunnday add one day
            $deliveryDays ++;
        }
        return date('Y-m-d', strtotime("+$deliveryDays days"));
    }

    /**
     * Check cart items for max delivery duration for complete delivery
     *
     * @param array $basket
     * @return int
     */
    private function getMaxDeliveryTime($basket)
    {
        $defaultReturn = 2;
        if (!$basket) {
            return $defaultReturn;
        }

        $config = $this->tsConfig->getSettings($this->shop->getId());
        $productAvailableDeliveryDuration   = $config['trustedShopsInStockDeliveryTime'];
        $productUnavailableDeliveryDuration = $config['trustedShopsNotInStockDeliveryTime'];

        if ($this->basketContainsUnavailableProducts($basket)) {
            if ($productUnavailableDeliveryDuration > 0) {
                return $productUnavailableDeliveryDuration;
            } else {
                $releaseDates = array();
                foreach ($basket as $article) {
                    if (empty($article['releasedate'])) {
                        $arrayDateKey = 0;
                    } else {
                        $arrayDateKey = strtotime($article['releasedate']);
                    }
                    $releaseDates[$arrayDateKey][] = $article;
                }

                $hightestKey = max(array_keys($releaseDates));
                if ($hightestKey == 0) {
                    return 100;
                }
                $lastDateArticles = $releaseDates[$hightestKey];

                $maxShippingtime = max(array_column($lastDateArticles, 'shippingtime'));
                if (empty($maxShippingtime)) {
                    $maxShippingtime = $defaultReturn;
                }

                $latestReleasedate = new DateTime(date('Y-m-d', $hightestKey));
                $todayDate = new DateTime(date('Y-m-d'));

                $timeTillReleasedate = $latestReleasedate->diff($todayDate)->days;

                return $timeTillReleasedate + $maxShippingtime;

            }
        }
        if ($productAvailableDeliveryDuration > 0) {
            switch (date('N')) {
                case 5: //freitag
                    return $productAvailableDeliveryDuration +2;
                    break;
                case 6: //samstag
                    return $productAvailableDeliveryDuration +1;
                    break;
                default: // sonntag bis donnerstag
                    return $productAvailableDeliveryDuration;
            }
        }

        $shippingTimes = array_column($basket, 'shippingtime');

        if (!$shippingTimes) {
            return $defaultReturn;
        }

        $maxDeliveryTime = (int) max($shippingTimes);

        if (!$maxDeliveryTime || empty($maxDeliveryTime)) {
            return $defaultReturn;
        }
        return $maxDeliveryTime;
    }

    /**
     * @param array $basket
     * @return bool
     */
    private function basketContainsUnavailableProducts($basket)
    {
        foreach ($basket as $article) {
            if ($article['instock'] === null) {
                continue;
            }
            // check for insufficent stock
            if ($article['instock'] <= 0) {
                return true;
            }
        }
        return false;
    }

    /**
     * @param $shopId
     * @return array
     */
    private function getConfig($shopId)
    {
        return $this->tsConfig->getTrustedShopBasicConfig($shopId);
    }

    /**
     * @param \Enlight_Event_EventArgs $args
     */
    public function onFrontendPostDispatch(\Enlight_Event_EventArgs $args)
    {
        /** @var $subject \Enlight_Controller_Action */
        $subject = $args->getSubject();
        $request = $subject->Request();

        $view = $subject->View();
        $shopId = $this->shop->getId();
        $templateVersion = $this->shop->getTemplate()->getVersion();
        $config = $this->getConfig($shopId);

        if ($request->getActionName() == 'finish') {
            $basket = $view->getAssign('sBasket');
            $config['deliveryDate'] = $this->getDeliveryDate($basket['content']);

        }

        //extend template
        $view->assign('sTrustedShops', $config);
        $view->assign('trustedShopsPluginVersion', $this->getPluginDescriptionInformation('currentVersion'));
        $this->template->addTemplateDir($this->pathVar . 'Views/');

        if(version_compare($templateVersion, 3, '<')) {
            $this->template->addTemplateDir($this->pathVar . 'Views/emotion/');
            $view->extendsTemplate('frontend/index/index_ts_trustbadge.tpl');
            $view->extendsTemplate('frontend/index/detail_ts_productsticker_sw4.tpl');
            $view->extendsTemplate('frontend/index/detail_ts_productwidget.tpl');
            $view->extendsTemplate('frontend/index/checkout_trustcard.tpl');
            $view->extendsTemplate('frontend/index/header_ts_customjQuery.tpl');
            $view->extendsTemplate('frontend/index/sidebar.tpl');
        } else {
            $this->template->addTemplateDir($this->pathVar . 'Views/Responsive');
        }
    }

    public function getPluginDescriptionInformation($field)
    {
        $info = json_decode(file_get_contents($this->pathVar . DIRECTORY_SEPARATOR . 'plugin.json'), true);
        if (isset($info[$field])) {
            return $info[$field];
        } else {
            throw new Exception('The plugin has an invalid description file. The following field is missing: ' . $field);
        }
    }
}